/**
 * @file   llnotificationslistener.h
 * @author Brad Kittenbrink
 * @date   2009-07-08
 * @brief  Wrap subset of LLNotifications API in event API for test scripts.
 * 
 * $LicenseInfo:firstyear=2009&license=viewergpl$
 * 
 * Copyright (c) 2009-2010, Linden Research, Inc.
 * 
 * Second Life Viewer Source Code
 * The source code in this file ("Source Code") is provided by Linden Lab
 * to you under the terms of the GNU General Public License, version 2.0
 * ("GPL"), unless you have obtained a separate licensing agreement
 * ("Other License"), formally executed by you and Linden Lab.  Terms of
 * the GPL can be found in doc/GPL-license.txt in this distribution, or
 * online at http://secondlife.com/developers/opensource/gplv2
 * 
 * There are special exceptions to the terms and conditions of the GPL as
 * it is applied to this Source Code. View the full text of the exception
 * in the file doc/FLOSS-exception.txt in this software distribution, or
 * online at
 * http://secondlife.com/developers/opensource/flossexception
 * 
 * By copying, modifying or distributing this software, you acknowledge
 * that you have read and understood your obligations described above,
 * and agree to abide by those obligations.
 * 
 * ALL LINDEN LAB SOURCE CODE IS PROVIDED "AS IS." LINDEN LAB MAKES NO
 * WARRANTIES, EXPRESS, IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY,
 * COMPLETENESS OR PERFORMANCE.
 * $/LicenseInfo$
 * 
 */

#ifndef LL_LLNOTIFICATIONSLISTENER_H
#define LL_LLNOTIFICATIONSLISTENER_H

#include "lleventapi.h"
#include "llnotificationptr.h"
#include <boost/shared_ptr.hpp>
#include <map>
#include <string>

class LLNotifications;
class LLSD;

class LLNotificationsListener : public LLEventAPI
{
public:
    LLNotificationsListener(LLNotifications & notifications);
    ~LLNotificationsListener();

private:
    void requestAdd(LLSD const & event_data) const;

	void NotificationResponder(const std::string& replypump, 
							   const LLSD& notification, 
							   const LLSD& response) const;

    void listChannels(const LLSD& params) const;
    void listChannelNotifications(const LLSD& params) const;
    void respond(const LLSD& params) const;
    void cancel(const LLSD& params) const;
    void ignore(const LLSD& params) const;
    void forward(const LLSD& params);

    static LLSD asLLSD(LLNotificationPtr);

    class Forwarder;
    typedef std::map<std::string, boost::shared_ptr<Forwarder> > ForwarderMap;
    ForwarderMap mForwarders;
	LLNotifications & mNotifications;
};

#endif // LL_LLNOTIFICATIONSLISTENER_H
